@@ -7,7 +7,7 @@
  * Rujeko Chinomona @ SMU.
  * ------------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2019, Lawrence Livermore National Security
+ * Copyright (c) 2002-2020, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -43,7 +43,8 @@
 /* Header files */
 #include <stdio.h>
 #include <math.h>
-#include <arkode/arkode_mristep.h>    /* prototypes for ARKStep fcts., consts */
+#include <arkode/arkode_mristep.h>    /* prototypes for MRIStep fcts., consts */
+#include <arkode/arkode_arkstep.h>    /* prototypes for ARKStep fcts., consts */
 #include <nvector/nvector_serial.h>   /* serial N_Vector types, fcts., macros */
 #include <sundials/sundials_types.h>  /* def. of type 'realtype'              */
 
@@ -78,7 +79,7 @@ int main()
   realtype Tf = RCONST(1.0);     /* final time */
   realtype dTout = RCONST(0.1);  /* time between outputs */
   sunindextype NEQ = 3;          /* number of dependent vars. */
-  int Nt = ceil(Tf/dTout);       /* number of output times */
+  int Nt = (int) ceil(Tf/dTout); /* number of output times */
   realtype hs = RCONST(0.001);   /* slow step size */
   realtype hf = RCONST(0.0001);  /* fast step size */
   realtype u0, v0, w0;           /* initial conditions */
@@ -88,11 +89,12 @@ int main()
   N_Vector y = NULL;             /* empty vector for the computed solution */
   N_Vector ytrue = NULL;         /* empty vector for the analytic solution */
   void *arkode_mem = NULL;       /* empty ARKode memory structure */
+  void *inner_arkode_mem = NULL; /* empty ARKode memory structure */
   FILE *UFID;
   realtype t, tout;
   realtype error = RCONST(0.0);
   int iout;
-  long int nsts, nstf, nfs, nff;
+  long int nsts, nstf, nfs, nff, tmp;
 
   /*
    * Initialization
@@ -116,20 +118,38 @@ int main()
   NV_Ith_S(y,2) = w0;
 
   /* Create serial vector for the analytic solution */
-  ytrue = N_VNew_Serial(NEQ);
-
-  /* Call MRIStepCreate to initialize the MRI timestepper module and
-     specify the right-hand side functions in y'=fs(t,y)+ff(t,y),
-     the inital time T0, and the initial dependent variable vector y. */
-  arkode_mem = MRIStepCreate(fs, ff, T0, y);
-  if (check_retval((void *)arkode_mem, "MRIStepCreate", 0)) return 1;
+  ytrue = N_VClone(y);
 
   /*
-   * Set integrator options
+   * Create the fast integrator and set options
    */
 
-  /* Specify slow and fast step sizes */
-  retval = MRIStepSetFixedStep(arkode_mem, hs, hf);
+  /* Initialize the fast integrator. Specify the fast right-hand side
+     function in y'=fs(t,y)+ff(t,y), the inital time T0, and the
+     initial dependent variable vector y. */
+  inner_arkode_mem = ARKStepCreate(ff, NULL, T0, y);
+  if (check_retval((void *) inner_arkode_mem, "ARKStepCreate", 0)) return 1;
+
+  /* Set the fast method */
+  retval = ARKStepSetTableNum(inner_arkode_mem, -1, KNOTH_WOLKE_3_3);
+  if (check_retval(&retval, "ARKStepSetTableNum", 1)) return 1;
+
+  /* Set the fast step size */
+  retval = ARKStepSetFixedStep(inner_arkode_mem, hf);
+  if (check_retval(&retval, "ARKStepSetFixedStep", 1)) return 1;
+
+  /*
+   * Create the slow integrator and set options
+   */
+
+  /* Initialize the slow integrator. Specify the slow right-hand side
+     function in y'=fs(t,y)+ff(t,y), the inital time T0, the
+     initial dependent variable vector y, and the fast integrator. */
+  arkode_mem = MRIStepCreate(fs, T0, y, MRISTEP_ARKSTEP, inner_arkode_mem);
+  if (check_retval((void *)arkode_mem, "MRIStepCreate", 0)) return 1;
+
+  /* Set the slow step size */
+  retval = MRIStepSetFixedStep(arkode_mem, hs);
   if (check_retval(&retval, "MRIStepSetFixedStep", 1)) return 1;
 
   /*
@@ -185,19 +205,28 @@ int main()
    * Finalize
    */
 
-  /* Print some final statistics */
-  retval = MRIStepGetNumSteps(arkode_mem, &nsts, &nstf);
+  /* Get some slow integrator statistics */
+  retval = MRIStepGetNumSteps(arkode_mem, &nsts);
+  check_retval(&retval, "MRIStepGetNumSteps", 1);
+  retval = MRIStepGetNumRhsEvals(arkode_mem, &nfs);
+  check_retval(&retval, "MRIStepGetNumRhsEvals", 1);
+
+  /* Get some fast integrator statistics */
+  retval = ARKStepGetNumSteps(inner_arkode_mem, &nstf);
   check_retval(&retval, "ARKStepGetNumSteps", 1);
-  retval = MRIStepGetNumRhsEvals(arkode_mem, &nfs, &nff);
+  retval = ARKStepGetNumRhsEvals(inner_arkode_mem, &nff, &tmp);
   check_retval(&retval, "ARKStepGetNumRhsEvals", 1);
 
+  /* Print some final statistics */
   printf("\nFinal Solver Statistics:\n");
   printf("   Steps: nsts = %li, nstf = %li\n", nsts, nstf);
   printf("   Total RHS evals:  Fs = %li,  Ff = %li\n", nfs, nff);
 
   /* Clean up and return */
-  N_VDestroy(y);            /* Free y vector */
-  MRIStepFree(&arkode_mem); /* Free integrator memory */
+  N_VDestroy(y);                  /* Free y vector */
+  N_VDestroy(ytrue);              /* Free ytrue vector */
+  ARKStepFree(&inner_arkode_mem); /* Free integrator memory */
+  MRIStepFree(&arkode_mem);       /* Free integrator memory */
 
   return 0;
 }
