@@ -2,15 +2,15 @@
  * Programmer(s): Allan Taylor, Alan Hindmarsh and
  *                Radu Serban @ LLNL
  * -----------------------------------------------------------------
- * LLNS Copyright Start
- * Copyright (c) 2017, Lawrence Livermore National Security
- * This work was performed under the auspices of the U.S. Department 
- * of Energy by Lawrence Livermore National Laboratory in part under 
- * Contract W-7405-Eng-48 and in part under Contract DE-AC52-07NA27344.
- * Produced at the Lawrence Livermore National Laboratory.
+ * SUNDIALS Copyright Start
+ * Copyright (c) 2002-2020, Lawrence Livermore National Security
+ * and Southern Methodist University.
  * All rights reserved.
- * For details, see the LICENSE file.
- * LLNS Copyright End
+ *
+ * See the top-level LICENSE and NOTICE files for details.
+ *
+ * SPDX-License-Identifier: BSD-3-Clause
+ * SUNDIALS Copyright End
  * -----------------------------------------------------------------
  *
  * This example loops through the available iterative linear solvers:
@@ -50,6 +50,18 @@
 #include <nvector/nvector_serial.h>      /* serial N_Vector types, fct. and macros */
 #include <sundials/sundials_types.h>     /* definition of realtype */
 
+/* helpful macros */
+
+#ifndef SQRT
+#if defined(SUNDIALS_DOUBLE_PRECISION)
+#define SQRT(x) (sqrt((x)))
+#elif defined(SUNDIALS_SINGLE_PRECISION)
+#define SQRT(x) (sqrtf((x)))
+#elif defined(SUNDIALS_EXTENDED_PRECISION)
+#define SQRT(x) (sqrtl((x)))
+#endif
+#endif
+
 /* Problem Constants */
 
 #define NOUT  11
@@ -103,7 +115,7 @@ static int check_retval(void *returnvalue, const char *funcname, int opt);
  *--------------------------------------------------------------------
  */
 
-int main(void)
+int main(int argc, char* argv[])
 {
   void *mem;
   UserData data;
@@ -112,11 +124,20 @@ int main(void)
   realtype rtol, atol, t0, t1, tout, tret;
   long int netf, ncfn, ncfl;
   SUNLinearSolver LS;
+  int nrmfactor;   /* LS norm conversion factor flag */
+  realtype nrmfac; /* LS norm conversion factor      */
 
-  mem = NULL;
-  data = NULL;
-  uu = up = constraints = res = NULL;
-  LS = NULL;
+  mem         = NULL;
+  data        = NULL;
+  uu          = NULL;
+  up          = NULL;
+  constraints = NULL;
+  res         = NULL;
+  LS          = NULL;
+  nrmfactor   = 0;
+
+  /* Retrieve the command-line options */
+  if (argc > 1) nrmfactor = atoi(argv[1]);
 
   /* Allocate N-vectors and the user data structure. */
 
@@ -259,6 +280,26 @@ int main(void)
     retval = IDASetPreconditioner(mem, PsetupHeat, PsolveHeat);
     if(check_retval(&retval, "IDASetPreconditioner", 1)) return(1);
 
+        /* Set the linear solver tolerance conversion factor */
+    switch(nrmfactor) {
+
+    case(1):
+      /* use the square root of the vector length */
+      nrmfac = SQRT((realtype)NEQ);
+      break;
+    case(2):
+      /* compute with dot product */
+      nrmfac = -ONE;
+      break;
+    default:
+      /* use the default */
+      nrmfac = ZERO;
+      break;
+    }
+
+    retval = IDASetLSNormFactor(mem, nrmfac);
+    if (check_retval(&retval, "IDASetLSNormFactor", 1)) return(1);
+
     /* Print output heading. */
     PrintHeader(rtol, atol, linsolver);
 
